function flatconv( scriptname )

local msg := {}

set exclusive off

SET cursor (.F.)

IF SCRIPTNAME = NIL
  msg:={'Usage:',;
        'FLATCONV ????????.CVO  (for flatfile output)',;
        'FLATCONV ????????.CVI  (for flatfile input) '}
  pop_msg(msg)
  IF verify('Do you need more help?')
    DO flathelp
  ENDIF
  QUIT
ENDIF

scriptname = UPPER(alltrim(scriptname))

IF FILE(scriptname)
  IF RIGHT(scriptname,1) = 'O'
    flat_out(scriptname)
  ENDIF
  IF RIGHT(scriptname,1) = 'I'
    flat_in(scriptname)
  ENDIF
ELSE
  pop_msg('Script '+scriptname+' does not exist!')
ENDIF

SET cursor (.T.)

RETURN nil

external strzero,D2C,C2D

FUNCTION flathelp

LOCAL MSG := {}

IF verify('Which type of help do you need?',.T.,'Output','Input')
  msg := {}
  aadd(msg,'You must create a script file to drive the conversion process')
  aadd(msg,'It must have the extension of .CVO')
  aadd(msg,'')
  aadd(msg,'The header must contain 5 lines of information.')
  aadd(msg,'')
  aadd(msg,'TEXTOUT = <YOUR TEXT OUTPUT FILE NAME>')
  aadd(msg,'DBFIN = <YOUR DBF TO SEND FROM>       ')
  aadd(msg,'INDEX = <ANY ASSOCIATED INDEXES>      ')
  aadd(msg,'DELIMITER = <YOUR DELIMITER>          ')
  aadd(msg, 'FIELDS = <NUMBER OF FIELD FUNCTIONS>  ')
  aadd(msg,'')
  aadd(msg,'Followed immediately by the functions or fieldname list')
  pop_msg(msg)
  msg := {}
  aadd(msg,'Example Script')
  aadd(msg,'')
  aadd(msg,'TEXTOUT = TESTDLIM.OUT    ')
  aadd(msg,'DBFIN = \INDEX2\INDEX2.DBF')
  aadd(msg,'INDEX = \INDEX2\INDXLNAM  ')
  aadd(msg,'DELIMITER = CHR(44)       ')
  aadd(msg,'FIELDS = 7                ')
  aadd(msg,'FNAME                     ')
  aadd(msg,'LNAME                     ')
  aadd(msg,'lj(ADDRESS,30)            ')
  aadd(msg,'space(14)                 ')
  aadd(msg,'CITY                      ')
  aadd(msg,'upper(STATE)              ')
  aadd(msg,'ZIP                       ')
  pop_msg(msg)
ELSE
  msg := {}
  aadd(msg,'You must create a script file to drive the conversion process')
  aadd(msg,'It must have the extension of .CVI')
  aadd(msg,'')
  aadd(msg,'The header must contain 5 lines of information.')
  aadd(msg,'')
  aadd(msg,'TEXTIN = <YOUR TEXT TO APPEND FROM> ')
  aadd(msg,'DBFOUT = <YOUR DBF OUTPUT FILE NAME>')
  aadd(msg,'INDEX = <ANY ASSOCIATED INDEXES>    ')
  aadd(msg,'DELIMITER = <YOUR DELIMITER>        ')
  aadd(msg,'FIELDS = <NUMBER OF FIELD FUNCTIONS>')
  aadd(msg,'')
  aadd(msg,'Followed immediately by the functions or fieldname list')
  pop_msg(msg)
  IF verify('Select your type of flat file',.T.,'Delimited','NON Delimited')
    msg := {}
    aadd(msg,'Example Script')
    aadd(msg,'')
    aadd(msg,'TEXTIN = TESTDLIM.IN         ')
    aadd(msg,'DBFOUT = TEST.DBF            ')
    aadd(msg,'INDEX = TEST                 ')
    aadd(msg,'DELIMITER = CHR(44)          ')
    aadd(msg,'FIELDS = 8                   ')
    aadd(msg,'1     FNAME     TOKEN        ')
    aadd(msg,'2     LNAME     TOKEN        ')
    aadd(msg,'3     ADDRESS   TOKEN        ')
    aadd(msg,'4     CITY      TOKEN        ')
    aadd(msg,'5     STATE     TOKEN        ')
    aadd(msg,'6     ZIP       LJ(TOKEN,10) ')
    aadd(msg,'7     NUM_TRYS  STR(TOKEN,10)')
    aadd(msg,'8     MDATE     CTOD(TOKEN)  ')
    aadd(msg,'')
    aadd(msg,'The word TOKEN should be placed in the formula where')
    aadd(msg,'you want the input text to be placed for processing.')
    aadd(msg,'It is up to you to convert all incoming variables to the correct')
    aadd(msg,'type. Flatfiles come in as text, and you must use a formula')
    aadd(msg,'in the third column if it will go in any other type of field.')
    pop_msg(msg)
  ELSE
    msg := {}
    aadd(msg,'Example Script')
    aadd(msg,'')
    aadd(msg,'TEXTIN = TESTDLIM.IN                 ')
    aadd(msg,'DBFOUT = TEST.DBF                    ')
    aadd(msg,'INDEX = TEST                         ')
    aadd(msg,'DELIMITER = POSITION                 ')
    aadd(msg,'FIELDS = 8                           ')
    aadd(msg,'001    010    FNAME     TOKEN        ')
    aadd(msg,'011    020    LNAME     TOKEN        ')
    aadd(msg,'021    040    ADDRESS   TOKEN        ')
    aadd(msg,'041    050    CITY      TOKEN        ')
    aadd(msg,'051    060    STATE     TOKEN        ')
    aadd(msg,'061    070    ZIP       LJ(TOKEN,10) ')
    aadd(msg,'071    080    NUM_TRYS  STR(TOKEN,10)')
    aadd(msg,'081    089    MDATE     CTOD(TOKEN)  ')
    aadd(msg,'')
    aadd(msg,'The word TOKEN should be placed in the formula where')
    aadd(msg,'you want the input text to be placed for processing.')
    aadd(msg,'It is up to you to convert all incoming variables to the correct')
    aadd(msg,'type. Flatfiles come in as text, and you must use a formula')
    aadd(msg,'in the fourth column if it will go in any other type of field.')
    pop_msg(msg)
  ENDIF
  msg := {}
  aadd(msg,'Any Clipper function may be used.')
  aadd(msg,'')
  aadd(msg,'The following functions have also been provided.')
  aadd(msg,'')
  aadd(msg,'LJ(<CHAR>,<NUM>)  Left justifies a string padded to <NUM> spaces')
  aadd(msg,'')
  aadd(msg,'RJ(<CHAR>,<NUM>)  Same as above, except right justified')
  aadd(msg,'')
  aadd(msg,'RJZF(<CHAR>,<NUM>) Same as RJ, Except ZERO FILLED')
  pop_msg(msg)
ENDIF

RETURN .T.

function jdtod(YYDOY)

// this function assumes an input of YYDDD where YY = YEAR and DDD = DOTY
// and return a date type

local monthsize := { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }
local year, doy, leap, test, month

IF EMPTY( VAL( YYDOY ) )
  RETURN ctod( "" )
ENDIF

year := left( YYDOY, 2 )
DOY := val( right( YYDOY, 3 ) )

leap := ( year % 4 ) == 0 .and. ( ( year % 100 ) <> 0 .or. ( year % 400 ) == 0 )

if leap .and. doy > 60
  doy --
endif

IF DOY < 1 .OR. DOY > 365
  DOY := 1
  MONTH := 1
ELSE
  for month := 1 to 11
    IF DOY > MONTHSIZE[ MONTH ]
      DOY -= MONTHSIZE[ MONTH ]
    ELSE
      EXIT
    ENDIF
  next
ENDIF

return ctod( strzero( month, 2 ) + "/" + strzero( doy, 2 ) + "/" + year )

FUNCTION dtojd( dInDate )

  LOCAL nDayNum, cCentury

  IF VALTYPE(dInDate) != "D"
     nDayNum := NIL
  ELSE

     // resolve century issue
     IF LEN( DTOC( dInDate) ) > 8                  // CENTURY is on
     cCentury := SUBSTR( DTOC( dInDate) ,7 ,4)
     ELSE
     cCentury := SUBSTR( DTOC( dInDate) ,7 ,2)
     END
  
     // calculate
     nDayNum := (dInDate - CTOD ("01/01/" + cCentury)) + 1

  END

RETURN (nDayNum)
